/* valascope.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_SCOPE (vala_scope_get_type ())
#define VALA_SCOPE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SCOPE, ValaScope))
#define VALA_SCOPE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SCOPE, ValaScopeClass))
#define VALA_IS_SCOPE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SCOPE))
#define VALA_IS_SCOPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SCOPE))
#define VALA_SCOPE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SCOPE, ValaScopeClass))

typedef struct _ValaScope ValaScope;
typedef struct _ValaScopeClass ValaScopeClass;
typedef struct _ValaScopePrivate ValaScopePrivate;

#define VALA_TYPE_CODE_NODE (vala_code_node_get_type ())
#define VALA_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CODE_NODE, ValaCodeNode))
#define VALA_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))
#define VALA_IS_CODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CODE_NODE))
#define VALA_IS_CODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CODE_NODE))
#define VALA_CODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CODE_NODE, ValaCodeNodeClass))

typedef struct _ValaCodeNode ValaCodeNode;
typedef struct _ValaCodeNodeClass ValaCodeNodeClass;

#define VALA_TYPE_SYMBOL (vala_symbol_get_type ())
#define VALA_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SYMBOL, ValaSymbol))
#define VALA_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SYMBOL, ValaSymbolClass))
#define VALA_IS_SYMBOL(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SYMBOL))
#define VALA_IS_SYMBOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SYMBOL))
#define VALA_SYMBOL_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SYMBOL, ValaSymbolClass))

typedef struct _ValaSymbol ValaSymbol;
typedef struct _ValaSymbolClass ValaSymbolClass;

#define VALA_TYPE_SOURCE_REFERENCE (vala_source_reference_get_type ())
#define VALA_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReference))
#define VALA_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))
#define VALA_IS_SOURCE_REFERENCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_SOURCE_REFERENCE))
#define VALA_IS_SOURCE_REFERENCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_SOURCE_REFERENCE))
#define VALA_SOURCE_REFERENCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_SOURCE_REFERENCE, ValaSourceReferenceClass))

typedef struct _ValaSourceReference ValaSourceReference;
typedef struct _ValaSourceReferenceClass ValaSourceReferenceClass;
typedef struct _ValaParamSpecScope ValaParamSpecScope;

/**
 * Represents a part of the symbol tree.
 */
struct _ValaScope {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaScopePrivate * priv;
};

struct _ValaScopeClass {
	GTypeClass parent_class;
	void (*finalize) (ValaScope *self);
};

struct _ValaScopePrivate {
	ValaSymbol* _owner;
	ValaScope* _parent_scope;
	GeeMap* symbol_table;
	GeeList* anonymous_members;
};

struct _ValaParamSpecScope {
	GParamSpec parent_instance;
};


static gpointer vala_scope_parent_class = NULL;

gpointer vala_scope_ref (gpointer instance);
void vala_scope_unref (gpointer instance);
GParamSpec* vala_param_spec_scope (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_scope (GValue* value, gpointer v_object);
gpointer vala_value_get_scope (const GValue* value);
GType vala_scope_get_type (void);
gpointer vala_code_node_ref (gpointer instance);
void vala_code_node_unref (gpointer instance);
GParamSpec* vala_param_spec_code_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_code_node (GValue* value, gpointer v_object);
gpointer vala_value_get_code_node (const GValue* value);
GType vala_code_node_get_type (void);
GType vala_symbol_get_type (void);
#define VALA_SCOPE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_SCOPE, ValaScopePrivate))
enum  {
	VALA_SCOPE_DUMMY_PROPERTY
};
void vala_scope_set_owner (ValaScope* self, ValaSymbol* value);
ValaScope* vala_scope_new (ValaSymbol* owner);
ValaScope* vala_scope_construct (GType object_type, ValaSymbol* owner);
ValaScope* vala_scope_new (ValaSymbol* owner);
ValaSymbol* vala_scope_lookup (ValaScope* self, const char* name);
void vala_code_node_set_error (ValaCodeNode* self, gboolean value);
ValaSymbol* vala_scope_get_owner (ValaScope* self);
gpointer vala_source_reference_ref (gpointer instance);
void vala_source_reference_unref (gpointer instance);
GParamSpec* vala_param_spec_source_reference (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_source_reference (GValue* value, gpointer v_object);
gpointer vala_value_get_source_reference (const GValue* value);
GType vala_source_reference_get_type (void);
void vala_report_error (ValaSourceReference* source, const char* message);
ValaSourceReference* vala_code_node_get_source_reference (ValaCodeNode* self);
char* vala_symbol_get_full_name (ValaSymbol* self);
void vala_symbol_set_owner (ValaSymbol* self, ValaScope* value);
void vala_scope_add (ValaScope* self, const char* name, ValaSymbol* sym);
void vala_scope_remove (ValaScope* self, const char* name);
gboolean vala_symbol_get_active (ValaSymbol* self);
ValaScope* vala_scope_get_parent_scope (ValaScope* self);
gboolean vala_scope_is_subscope_of (ValaScope* self, ValaScope* scope);
GeeMap* vala_scope_get_symbol_table (ValaScope* self);
void vala_scope_set_parent_scope (ValaScope* self, ValaScope* value);
static void vala_scope_finalize (ValaScope* obj);



/**
 * Creates a new scope.
 *
 * @return newly created scope
 */
ValaScope* vala_scope_construct (GType object_type, ValaSymbol* owner) {
	ValaScope* self;
	self = (ValaScope*) g_type_create_instance (object_type);
	vala_scope_set_owner (self, owner);
	return self;
}


ValaScope* vala_scope_new (ValaSymbol* owner) {
	return vala_scope_construct (VALA_TYPE_SCOPE, owner);
}


/**
 * Adds the specified symbol with the specified name to the symbol table
 * of this scope.
 *
 * @param name name for the specified symbol
 * @param sym  a symbol
 */
void vala_scope_add (ValaScope* self, const char* name, ValaSymbol* sym) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sym != NULL);
	if (name != NULL) {
		if (self->priv->symbol_table == NULL) {
			GeeMap* _tmp0_;
			_tmp0_ = NULL;
			self->priv->symbol_table = (_tmp0_ = (GeeMap*) gee_hash_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_str_hash, g_str_equal, g_direct_equal), (self->priv->symbol_table == NULL) ? NULL : (self->priv->symbol_table = (gee_collection_object_unref (self->priv->symbol_table), NULL)), _tmp0_);
		} else {
			ValaSymbol* _tmp1_;
			gboolean _tmp2_;
			_tmp1_ = NULL;
			if ((_tmp2_ = (_tmp1_ = vala_scope_lookup (self, name)) != NULL, (_tmp1_ == NULL) ? NULL : (_tmp1_ = (vala_code_node_unref (_tmp1_), NULL)), _tmp2_)) {
				char* _tmp4_;
				char* _tmp3_;
				vala_code_node_set_error ((ValaCodeNode*) self->priv->_owner, TRUE);
				_tmp4_ = NULL;
				_tmp3_ = NULL;
				vala_report_error (vala_code_node_get_source_reference ((ValaCodeNode*) sym), _tmp4_ = g_strdup_printf ("`%s' already contains a definition for `%s'", _tmp3_ = vala_symbol_get_full_name (self->priv->_owner), name));
				_tmp4_ = (g_free (_tmp4_), NULL);
				_tmp3_ = (g_free (_tmp3_), NULL);
				return;
			}
		}
		gee_map_set (self->priv->symbol_table, (const char*) name, sym);
	} else {
		if (self->priv->anonymous_members == NULL) {
			GeeList* _tmp5_;
			_tmp5_ = NULL;
			self->priv->anonymous_members = (_tmp5_ = (GeeList*) gee_array_list_new (VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, g_direct_equal), (self->priv->anonymous_members == NULL) ? NULL : (self->priv->anonymous_members = (gee_collection_object_unref (self->priv->anonymous_members), NULL)), _tmp5_);
		}
		gee_collection_add ((GeeCollection*) self->priv->anonymous_members, sym);
	}
	vala_symbol_set_owner (sym, self);
}


void vala_scope_remove (ValaScope* self, const char* name) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);
	gee_map_remove (self->priv->symbol_table, name);
}


/**
 * Returns the symbol stored in the symbol table with the specified
 * name.
 *
 * @param name name of the symbol to be returned
 * @return     found symbol or null
 */
ValaSymbol* vala_scope_lookup (ValaScope* self, const char* name) {
	ValaSymbol* result;
	ValaSymbol* sym;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	if (self->priv->symbol_table == NULL) {
		result = NULL;
		return result;
	}
	sym = (ValaSymbol*) gee_map_get (self->priv->symbol_table, name);
	_tmp0_ = FALSE;
	if (sym != NULL) {
		_tmp0_ = !vala_symbol_get_active (sym);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		ValaSymbol* _tmp1_;
		_tmp1_ = NULL;
		sym = (_tmp1_ = NULL, (sym == NULL) ? NULL : (sym = (vala_code_node_unref (sym), NULL)), _tmp1_);
	}
	result = sym;
	return result;
}


/**
 * Returns whether the specified scope is an ancestor of this scope.
 *
 * @param scope a scope or null for the root scope
 * @return      true if this scope is a subscope of the specified
 *              scope, false otherwise
 */
gboolean vala_scope_is_subscope_of (ValaScope* self, ValaScope* scope) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	if (scope == self) {
		result = TRUE;
		return result;
	}
	/* null scope is the root scope*/
	if (scope == NULL) {
		result = TRUE;
		return result;
	}
	if (self->priv->_parent_scope != NULL) {
		result = vala_scope_is_subscope_of (self->priv->_parent_scope, scope);
		return result;
	}
	result = FALSE;
	return result;
}


GeeMap* vala_scope_get_symbol_table (ValaScope* self) {
	GeeMap* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GeeMap*) gee_read_only_map_new (G_TYPE_STRING, (GBoxedCopyFunc) g_strdup, g_free, VALA_TYPE_SYMBOL, (GBoxedCopyFunc) vala_code_node_ref, vala_code_node_unref, self->priv->symbol_table);
	return result;
}


ValaSymbol* vala_scope_get_owner (ValaScope* self) {
	ValaSymbol* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_owner;
	return result;
}


void vala_scope_set_owner (ValaScope* self, ValaSymbol* value) {
	g_return_if_fail (self != NULL);
	self->priv->_owner = value;
}


ValaScope* vala_scope_get_parent_scope (ValaScope* self) {
	ValaScope* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_parent_scope;
	return result;
}


void vala_scope_set_parent_scope (ValaScope* self, ValaScope* value) {
	g_return_if_fail (self != NULL);
	self->priv->_parent_scope = value;
}


static void vala_value_scope_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_scope_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_scope_unref (value->data[0].v_pointer);
	}
}


static void vala_value_scope_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_scope_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_scope_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_scope_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaScope* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_scope_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_scope_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaScope** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_scope_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_scope (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecScope* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_SCOPE), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_scope (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_SCOPE), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_scope (GValue* value, gpointer v_object) {
	ValaScope* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_SCOPE));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_SCOPE));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_scope_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_scope_unref (old);
	}
}


static void vala_scope_class_init (ValaScopeClass * klass) {
	vala_scope_parent_class = g_type_class_peek_parent (klass);
	VALA_SCOPE_CLASS (klass)->finalize = vala_scope_finalize;
	g_type_class_add_private (klass, sizeof (ValaScopePrivate));
}


static void vala_scope_instance_init (ValaScope * self) {
	self->priv = VALA_SCOPE_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_scope_finalize (ValaScope* obj) {
	ValaScope * self;
	self = VALA_SCOPE (obj);
	(self->priv->symbol_table == NULL) ? NULL : (self->priv->symbol_table = (gee_collection_object_unref (self->priv->symbol_table), NULL));
	(self->priv->anonymous_members == NULL) ? NULL : (self->priv->anonymous_members = (gee_collection_object_unref (self->priv->anonymous_members), NULL));
}


GType vala_scope_get_type (void) {
	static GType vala_scope_type_id = 0;
	if (vala_scope_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_scope_init, vala_value_scope_free_value, vala_value_scope_copy_value, vala_value_scope_peek_pointer, "p", vala_value_scope_collect_value, "p", vala_value_scope_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaScopeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_scope_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaScope), 0, (GInstanceInitFunc) vala_scope_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_scope_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaScope", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vala_scope_type_id;
}


gpointer vala_scope_ref (gpointer instance) {
	ValaScope* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_scope_unref (gpointer instance) {
	ValaScope* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_SCOPE_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




